use wccg_models::event::ControlMarking;
use wread_mongodb::crud_repository;
use wread_mongodb::mongodb::bson::{doc, oid::ObjectId};
use wread_mongodb::mongodb::error::Error;
use wread_mongodb::mongodb::options::ReplaceOptions;
use wread_mongodb::mongodb::results::{DeleteResult, InsertOneResult, UpdateResult};
use wread_mongodb::mongodb::Database;

const COLLECTION_NAME: &str = "controlMarkings";

pub async fn get(id: &str, db: &Database) -> Result<Option<ControlMarking>, Error> {
    let id_object = ObjectId::with_string(id).unwrap();
    crud_repository::find_by_id(&id_object, COLLECTION_NAME, &db).await
}

pub async fn get_by_rider_id(rider_id: &str, db: &Database) -> Result<Vec<ControlMarking>, Error> {
    let filter = doc! {"riderId": rider_id };
    crud_repository::find(filter, COLLECTION_NAME, &db).await
}

pub async fn get_by_event_id(event_id: &str, db: &Database) -> Result<Vec<ControlMarking>, Error> {
    let filter = doc! {"eventId": event_id };
    crud_repository::find(filter, COLLECTION_NAME, &db).await
}

pub async fn get_by_rider_id_for_event(
    rider_id: &str,
    event_id: &str,
    db: &Database,
) -> Result<Vec<ControlMarking>, Error> {
    let filter = doc! {"riderId": rider_id, "eventId": event_id };
    crud_repository::find(filter, COLLECTION_NAME, &db).await
}

pub async fn get_by_rider_id_for_registration(
    rider_id: &str,
    event_id: &str,
    registration_option_id: &str,
    db: &Database,
) -> Result<Vec<ControlMarking>, Error> {
    let filter = doc! {"riderId": rider_id, "eventId": event_id, "registrationOptionId": registration_option_id };
    crud_repository::find(filter, COLLECTION_NAME, &db).await
}

pub async fn get_by_rider_id_for_control(
    rider_id: &str,
    event_id: &str,
    registration_option_id: &str,
    control_id: &str,
    db: &Database,
) -> Result<Option<ControlMarking>, Error> {
    let filter = doc! {"riderId": rider_id, "eventId": event_id, "registrationOptionId": registration_option_id, "controlId": control_id };
    crud_repository::find_one(filter, COLLECTION_NAME, &db).await
}

pub async fn add(marking: &ControlMarking, db: &Database) -> Result<InsertOneResult, Error> {
    crud_repository::add(marking, COLLECTION_NAME, db).await
}

pub async fn delete(id: &str, db: &Database) -> Result<DeleteResult, Error> {
    let id_object = ObjectId::with_string(id).unwrap();
    let query = doc! {"_id": id_object };
    crud_repository::delete_one(query, None, COLLECTION_NAME, &db).await
}

pub async fn update(marking: ControlMarking, db: &Database) -> Result<UpdateResult, Error> {
    let id = marking.id().clone().unwrap();
    let filter = doc! {"_id":  id};
    crud_repository::replace_one(filter, &marking, None, COLLECTION_NAME, db).await
}

pub async fn upsert(marking: ControlMarking, db: &Database) -> Result<UpdateResult, Error> {
    let query = doc! {"riderId": marking.rider_id(), "eventId": marking.event_id(), "registrationOptionId": marking.registration_option_id(), "controlId": marking.control_id() };
    let replace_options = ReplaceOptions::builder().upsert(true).build();
    crud_repository::replace_one(query, &marking, Some(replace_options), COLLECTION_NAME, db).await
}
